@@ -68,6 +68,67 @@
  * a #ClutterAnimation instance and animate an actor between its current
  * state and the specified final state.
  *
+ * <refsect2 id="clutter-AnimationMode-Script">
+ *   <title>Defining ClutterAnimationMode inside ClutterScript</title>
+ *   <para>When defining a #ClutterAnimation inside a ClutterScript
+ *   file or string the #ClutterAnimation:mode can be defined either
+ *   using the #ClutterAnimationMode enumeration values through their
+ *   "nick" (the short string used inside #GEnumValue), their numeric
+ *   id, or using the following strings:</para>
+ *   <variablelist>
+ *     <varlistentry>
+ *       <term>easeInQuad, easeOutQuad, easeInOutQuad</term>
+ *       <listitem><para>Corresponding to the quadratic easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInCubic, easeOutCubic, easeInOutCubic</term>
+ *       <listitem><para>Corresponding to the cubic easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInQuart, easeOutQuart, easeInOutQuart</term>
+ *       <listitem><para>Corresponding to the quartic easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInQuint, easeOutQuint, easeInOutQuint</term>
+ *       <listitem><para>Corresponding to the quintic easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInSine, easeOutSine, easeInOutSine</term>
+ *       <listitem><para>Corresponding to the sine easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInExpo, easeOutExpo, easeInOutExpo</term>
+ *       <listitem><para>Corresponding to the exponential easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInCirc, easeOutCirc, easeInOutCirc</term>
+ *       <listitem><para>Corresponding to the circular easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInElastic, easeOutElastic, easeInOutElastic</term>
+ *       <listitem><para>Corresponding to the overshooting elastic
+ *       easing modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInBack, easeOutBack, easeInOutBack</term>
+ *       <listitem><para>Corresponding to the overshooting cubic
+ *       easing modes</para></listitem>
+ *     </varlistentry>
+ *     <varlistentry>
+ *       <term>easeInBounce, easeOutBounce, easeInOutBounce</term>
+ *       <listitem><para>Corresponding to the bouncing easing
+ *       modes</para></listitem>
+ *     </varlistentry>
+ *   </variablelist>
+ * </refsect2>
+ *
  * #ClutterAnimation is available since Clutter 1.0
  */
 
@@ -85,6 +146,8 @@
 #include "clutter-enum-types.h"
 #include "clutter-interval.h"
 #include "clutter-private.h"
+#include "clutter-scriptable.h"
+#include "clutter-script-private.h"
 
 enum
 {
@@ -125,17 +188,41 @@ static guint animation_signals[LAST_SIGNAL] = { 0, };
 
 static GQuark quark_object_animation = 0;
 
-G_DEFINE_TYPE (ClutterAnimation, clutter_animation, G_TYPE_OBJECT);
+static void clutter_scriptable_init (ClutterScriptableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (ClutterAnimation, clutter_animation, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_SCRIPTABLE,
+                                                clutter_scriptable_init));
+
+static void
+on_actor_dispose (gpointer  user_data,
+                  GObject  *actor_pointer)
+{
+  ClutterAnimation *self = user_data;
+
+  if (self->priv->object == actor_pointer)
+    {
+      CLUTTER_NOTE (ANIMATION, "Object [%p] was unref'd", actor_pointer);
+      g_object_unref (self);
+    }
+}
+
 
 static void
 clutter_animation_real_completed (ClutterAnimation *self)
 {
   ClutterAnimationPrivate *priv = self->priv;
   ClutterAnimation *animation;
+  ClutterTimeline *timeline;
+  ClutterTimelineDirection direction;
   gpointer key, value;
   GHashTableIter iter;
 
+  timeline = clutter_animation_get_timeline (self);
+  direction = clutter_timeline_get_direction (timeline);
+
   /* explicitly set the final state of the animation */
+  CLUTTER_NOTE (ANIMATION, "Set final state on object [%p]", priv->object);
   g_hash_table_iter_init (&iter, priv->properties);
   while (g_hash_table_iter_next (&iter, &key, &value))
     {
@@ -143,7 +230,11 @@ clutter_animation_real_completed (ClutterAnimation *self)
       ClutterInterval *interval = value;
       GValue *p_value;
 
-      p_value = clutter_interval_peek_final_value (interval);
+      if (direction == CLUTTER_TIMELINE_FORWARD)
+        p_value = clutter_interval_peek_final_value (interval);
+      else
+        p_value = clutter_interval_peek_initial_value (interval);
+
       g_object_set_property (priv->object, p_name, p_value);
     }
 
@@ -158,7 +249,14 @@ clutter_animation_real_completed (ClutterAnimation *self)
   animation = g_object_get_qdata (priv->object, quark_object_animation);
   if (animation == self)
     {
+      CLUTTER_NOTE (ANIMATION, "Unsetting animation for actor [%p]",
+                    priv->object);
+
       g_object_set_qdata (priv->object, quark_object_animation, NULL);
+      g_object_weak_unref (priv->object, on_actor_dispose, self);
+
+      CLUTTER_NOTE (ANIMATION, "Releasing the reference Animation [%p]",
+                    animation);
       g_object_unref (animation);
     }
 }
@@ -168,7 +266,9 @@ clutter_animation_finalize (GObject *gobject)
 {
   ClutterAnimationPrivate *priv = CLUTTER_ANIMATION (gobject)->priv;
 
-  CLUTTER_NOTE (ANIMATION, "Destroying properties hash table");
+  CLUTTER_NOTE (ANIMATION,
+                "Destroying properties table for Animation [%p]",
+                gobject);
   g_hash_table_destroy (priv->properties);
 
   G_OBJECT_CLASS (clutter_animation_parent_class)->finalize (gobject);
@@ -294,6 +394,34 @@ clutter_animation_get_property (GObject    *gobject,
     }
 }
 
+static gboolean
+clutter_animation_parse_custom_node (ClutterScriptable *scriptable,
+                                     ClutterScript     *script,
+                                     GValue            *value,
+                                     const gchar       *name,
+                                     JsonNode          *node)
+{
+  if (strncmp (name, "mode", 4) == 0)
+    {
+      gulong mode;
+
+      mode = clutter_script_resolve_animation_mode (node);
+
+      g_value_init (value, G_TYPE_ULONG);
+      g_value_set_ulong (value, mode);
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+
+static void
+clutter_scriptable_init (ClutterScriptableIface *iface)
+{
+  iface->parse_custom_node = clutter_animation_parse_custom_node;
+}
+
 static void
 clutter_animation_class_init (ClutterAnimationClass *klass)
 {
@@ -313,7 +441,7 @@ clutter_animation_class_init (ClutterAnimationClass *klass)
   gobject_class->finalize = clutter_animation_finalize;
 
   /**
-   * ClutterAnimation:objct:
+   * ClutterAnimation:object:
    *
    * The #GObject to which the animation applies.
    *
@@ -498,6 +626,7 @@ clutter_animation_validate_bind (ClutterAnimation *animation,
   ClutterAnimationPrivate *priv;
   GObjectClass *klass;
   GParamSpec *pspec;
+  GType pspec_type;
 
   priv = animation->priv;
 
@@ -536,17 +665,20 @@ clutter_animation_validate_bind (ClutterAnimation *animation,
       return NULL;
     }
 
-  if (!g_value_type_compatible (G_PARAM_SPEC_VALUE_TYPE (pspec),
-                                argtype))
+  pspec_type = G_PARAM_SPEC_VALUE_TYPE (pspec);
+
+  if (!g_value_type_compatible (argtype, pspec_type) ||
+      !g_value_type_transformable (argtype, pspec_type))
     {
       g_warning ("Cannot bind property '%s': the interval value of "
                  "type '%s' is not compatible with the property value "
                  "of type '%s'",
                  property_name,
                  g_type_name (argtype),
-                 g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
+                 g_type_name (pspec_type));
       return NULL;
     }
+
   return pspec;
 }
 
@@ -603,6 +735,7 @@ clutter_animation_bind_interval (ClutterAnimation *animation,
  * calls convenient in language bindings.
  *
  * Return value: (transfer none): The animation itself.
+ *
  * Since: 1.0
  */
 ClutterAnimation *
@@ -622,8 +755,7 @@ clutter_animation_bind (ClutterAnimation *animation,
   priv = animation->priv;
 
   type = G_VALUE_TYPE (final);
-  pspec = clutter_animation_validate_bind (animation, property_name,
-                                           type);
+  pspec = clutter_animation_validate_bind (animation, property_name, type);
   if (pspec == NULL)
     return NULL;
 
@@ -715,6 +847,7 @@ clutter_animation_update_interval (ClutterAnimation *animation,
   ClutterAnimationPrivate *priv;
   GObjectClass *klass;
   GParamSpec *pspec;
+  GType pspec_type, int_type;
 
   g_return_if_fail (CLUTTER_IS_ANIMATION (animation));
   g_return_if_fail (property_name != NULL);
@@ -724,7 +857,7 @@ clutter_animation_update_interval (ClutterAnimation *animation,
 
   if (!clutter_animation_has_property (animation, property_name))
     {
-      g_warning ("Cannot unbind property '%s': the animation has "
+      g_warning ("Cannot update property '%s': the animation has "
                  "no bound property with that name",
                  property_name);
       return;
@@ -734,28 +867,84 @@ clutter_animation_update_interval (ClutterAnimation *animation,
   pspec = g_object_class_find_property (klass, property_name);
   if (!pspec)
     {
-      g_warning ("Cannot bind property '%s': objects of type '%s' have "
+      g_warning ("Cannot update property '%s': objects of type '%s' have "
                  "no such property",
                  property_name,
                  g_type_name (G_OBJECT_TYPE (priv->object)));
       return;
     }
 
-  if (!g_value_type_compatible (G_PARAM_SPEC_VALUE_TYPE (pspec),
-                                clutter_interval_get_value_type (interval)))
+  pspec_type = G_PARAM_SPEC_VALUE_TYPE (pspec);
+  int_type = clutter_interval_get_value_type (interval);
+
+  if (!g_value_type_compatible (int_type, pspec_type) ||
+      !g_value_type_transformable (int_type, pspec_type))
     {
-      g_warning ("Cannot bind property '%s': the interval value of "
+      g_warning ("Cannot update property '%s': the interval value of "
                  "type '%s' is not compatible with the property value "
                  "of type '%s'",
                  property_name,
-                 g_type_name (clutter_interval_get_value_type (interval)),
-                 g_type_name (G_PARAM_SPEC_VALUE_TYPE (pspec)));
+                 g_type_name (int_type),
+                 g_type_name (pspec_type));
       return;
     }
 
   clutter_animation_update_property_internal (animation, pspec, interval);
 }
 
+/**
+ * clutter_animation_update:
+ * @animation: a #ClutterAnimation
+ * @property_name: name of the property
+ * @final: The final value of the property
+ *
+ * Updates the @final value of the interval for @property_name
+ *
+ * Return value: (transfer none): The animation itself.
+ *
+ * Since: 1.0
+ */
+ClutterAnimation *
+clutter_animation_update (ClutterAnimation *animation,
+                          const gchar      *property_name,
+                          const GValue     *final)
+{
+  ClutterInterval *interval;
+  GType int_type;
+
+  g_return_val_if_fail (CLUTTER_IS_ANIMATION (animation), NULL);
+  g_return_val_if_fail (property_name != NULL, NULL);
+  g_return_val_if_fail (final != NULL, NULL);
+  g_return_val_if_fail (G_VALUE_TYPE (final) != G_TYPE_INVALID, NULL);
+
+  interval = clutter_animation_get_interval (animation, property_name);
+  if (interval == NULL)
+    {
+      g_warning ("Cannot update property '%s': the animation has "
+                 "no bound property with that name",
+                 property_name);
+      return NULL;
+    }
+
+  int_type = clutter_interval_get_value_type (interval);
+
+  if (!g_value_type_compatible (G_VALUE_TYPE (final), int_type) ||
+      !g_value_type_transformable (G_VALUE_TYPE (final), int_type))
+    {
+      g_warning ("Cannot update property '%s': the interval value of "
+                 "type '%s' is not compatible with the property value "
+                 "of type '%s'",
+                 property_name,
+                 g_type_name (int_type),
+                 g_type_name (G_VALUE_TYPE (final)));
+      return NULL;
+    }
+
+  clutter_interval_set_final_value (interval, final);
+
+  return animation;
+}
+
 /**
  * clutter_animation_get_interval:
  * @animation: a #ClutterAnimation
@@ -1037,6 +1226,8 @@ clutter_animation_set_mode (ClutterAnimation *animation,
   g_object_freeze_notify (G_OBJECT (animation));
 
   alpha = clutter_animation_get_alpha_internal (animation);
+  g_assert (CLUTTER_IS_ALPHA (alpha));
+
   clutter_alpha_set_mode (alpha, mode);
 
   g_object_notify (G_OBJECT (animation), "mode");
@@ -1090,6 +1281,8 @@ clutter_animation_set_duration (ClutterAnimation *animation,
   g_object_freeze_notify (G_OBJECT (animation));
 
   timeline = clutter_animation_get_timeline_internal (animation);
+  g_assert (CLUTTER_IS_TIMELINE (timeline));
+
   clutter_timeline_set_duration (timeline, msecs);
   clutter_timeline_rewind (timeline);
 
@@ -1340,7 +1533,12 @@ clutter_animation_set_alpha (ClutterAnimation *animation,
                           animation);
     }
   else
-    timeline = clutter_animation_get_timeline_internal (animation);
+    {
+      /* FIXME - add a create_timeline_internal() because this does
+       * not look very good
+       */
+      (void) clutter_animation_get_timeline_internal (animation);
+    }
 
 out:
   /* emit all relevant notifications */
@@ -1450,35 +1648,37 @@ clutter_animation_setup_property (ClutterAnimation *animation,
    */
   if (!g_type_is_a (G_VALUE_TYPE (value), G_VALUE_TYPE (&real_value)))
     {
-      if (!g_value_type_compatible (G_VALUE_TYPE (value),
-                                    G_VALUE_TYPE (&real_value)) &&
-          !g_value_type_compatible (G_VALUE_TYPE (&real_value),
-                                    G_VALUE_TYPE (value)))
+      /* are these two types compatible (can be directly copied)? */
+      if (g_value_type_compatible (G_VALUE_TYPE (value),
+                                   G_VALUE_TYPE (&real_value)))
         {
-          g_warning ("%s: Unable to convert from %s to %s for "
-                     "the property '%s' of object %s",
-                     G_STRLOC,
-                     g_type_name (G_VALUE_TYPE (value)),
-                     g_type_name (G_VALUE_TYPE (&real_value)),
-                     property_name,
-                     G_OBJECT_TYPE_NAME (priv->object));
-          g_value_unset (&real_value);
-          return;
+          g_value_copy (value, &real_value);
+          goto done;
         }
 
-      if (!g_value_transform (value, &real_value))
+      /* are these two type transformable? */
+      if (g_value_type_transformable (G_VALUE_TYPE (value),
+                                      G_VALUE_TYPE (&real_value)))
         {
-          g_warning ("%s: Unable to transform from %s to %s",
-                     G_STRLOC,
-                     g_type_name (G_VALUE_TYPE (value)),
-                     g_type_name (G_VALUE_TYPE (&real_value)));
-          g_value_unset (&real_value);
-          return;
+          if (g_value_transform (value, &real_value))
+            goto done;
         }
+
+      /* if not compatible and not transformable then we can't do much */
+      g_warning ("%s: Unable to convert from %s to %s for "
+                 "the property '%s' of object %s",
+                 G_STRLOC,
+                 g_type_name (G_VALUE_TYPE (value)),
+                 g_type_name (G_VALUE_TYPE (&real_value)),
+                 property_name,
+                 G_OBJECT_TYPE_NAME (priv->object));
+      g_value_unset (&real_value);
+      return;
     }
   else
     g_value_copy (value, &real_value);
 
+done:
   /* create an interval and bind it to the property, in case
    * it's not a fixed property, otherwise just set it
    */
@@ -1553,6 +1753,39 @@ clutter_animation_setupv (ClutterAnimation    *animation,
     }
 }
 
+static const struct
+{
+  const gchar *name;
+  GConnectFlags flags;
+} signal_prefixes[] =
+  {
+    { "::", 0 },
+    { "-swapped::", G_CONNECT_SWAPPED },
+    { "-after::", G_CONNECT_AFTER },
+    { "-swapped-after::", G_CONNECT_SWAPPED | G_CONNECT_AFTER }
+  };
+
+static gboolean
+clutter_animation_has_signal_prefix (const gchar *property_name,
+                                     GConnectFlags *flags,
+                                     int *offset)
+{
+  int i;
+
+  if (!g_str_has_prefix (property_name, "signal"))
+    return FALSE;
+
+  for (i = 0; i < G_N_ELEMENTS (signal_prefixes); i++)
+    if (g_str_has_prefix (property_name + 6, signal_prefixes[i].name))
+      {
+        *offset = strlen (signal_prefixes[i].name) + 6;
+        *flags = signal_prefixes[i].flags;
+        return TRUE;
+      }
+
+  return FALSE;
+}
+
 static void
 clutter_animation_setup_valist (ClutterAnimation *animation,
                                 const gchar      *first_property_name,
@@ -1571,46 +1804,20 @@ clutter_animation_setup_valist (ClutterAnimation *animation,
       GValue final = { 0, };
       gchar *error = NULL;
       gboolean is_fixed = FALSE;
+      GConnectFlags flags;
+      int offset;
 
-      if (g_str_has_prefix (property_name, "signal::"))
+      if (clutter_animation_has_signal_prefix (property_name,
+                                               &flags,
+                                               &offset))
         {
-          const gchar *signal_name = property_name + 8;
+          const gchar *signal_name = property_name + offset;
           GCallback callback = va_arg (var_args, GCallback);
           gpointer  userdata = va_arg (var_args, gpointer);
 
-          g_signal_connect (animation, signal_name, callback, userdata);
-        }
-      else if (g_str_has_prefix (property_name, "signal-"))
-        {
-          GCallback callback = va_arg (var_args, GCallback);
-          gpointer  userdata = va_arg (var_args, gpointer);
-          const gchar *signal_name;
-          GConnectFlags flags;
-
-          if (g_str_has_prefix (property_name, "signal-after::"))
-            {
-              signal_name = property_name + 14;
-              flags = G_CONNECT_AFTER;
-            }
-          else if (g_str_has_prefix (property_name, "signal-swapped::"))
-            {
-              signal_name = property_name + 16;
-              flags = G_CONNECT_SWAPPED;
-            }
-          else
-            {
-              g_warning ("Unable to connect to '%s': the valid signal "
-                         "modifiers are 'signal-swapped::' and "
-                         "'signal-swapped::'",
-                         property_name);
-              break;
-            }
-
           g_signal_connect_data (animation, signal_name,
-                                 callback,
-                                 userdata,
-                                 NULL,
-                                 flags);
+                                 callback, userdata,
+                                 NULL, flags);
         }
       else
         {
@@ -1630,8 +1837,16 @@ clutter_animation_setup_valist (ClutterAnimation *animation,
               break;
             }
 
+#if GLIB_CHECK_VERSION (2, 23, 2)
+          G_VALUE_COLLECT_INIT (&final, G_PARAM_SPEC_VALUE_TYPE (pspec),
+                                var_args, 0,
+                                &error);
+#else
+          /* this is the same as G_VALUE_COLLECT_INIT(), but slower */
           g_value_init (&final, G_PARAM_SPEC_VALUE_TYPE (pspec));
           G_VALUE_COLLECT (&final, var_args, 0, &error);
+#endif /* GLIB_CHECK_VERSION (2, 23, 2) */
+
           if (error)
             {
               g_warning ("%s: %s", G_STRLOC, error);
@@ -1661,6 +1876,7 @@ animation_create_for_actor (ClutterActor *actor)
       animation = clutter_animation_new ();
       clutter_animation_set_object (animation, object);
       g_object_set_qdata (object, quark_object_animation, animation);
+      g_object_weak_ref (object, on_actor_dispose, animation);
 
       CLUTTER_NOTE (ANIMATION,
                     "Created new Animation [%p] for actor [%p]",
@@ -1802,8 +2018,8 @@ clutter_actor_animate_with_timeline (ClutterActor    *actor,
  *
  * |[
  *   clutter_actor_animate (rectangle, CLUTTER_LINEAR, 250,
- *                          "width", 100,
- *                          "height", 100,
+ *                          "width", 100.0,
+ *                          "height", 100.0,
  *                          NULL);
  * ]|
  *
@@ -1819,8 +2035,8 @@ clutter_actor_animate_with_timeline (ClutterActor    *actor,
  * by the "fixed::" string, for instance:
  *
  * |[
- *   clutter_actor_animate (actor, CLUTTER_EASE_IN, 100,
- *                          "rotation-angle-z", 360,
+ *   clutter_actor_animate (actor, CLUTTER_EASE_IN_SINE, 100,
+ *                          "rotation-angle-z", 360.0,
  *                          "fixed::rotation-center-z", &amp;center,
  *                          NULL);
  * ]|
@@ -1834,11 +2050,10 @@ clutter_actor_animate_with_timeline (ClutterActor    *actor,
  * to control the animation or to know when the animation has been
  * completed.
  *
- * If a name argument starts with "signal::", "signal-after::" or
- * "signal-swapped::" the two following arguments are used as callback
- * function and data for a signal handler installed on the
- * #ClutterAnimation object for the specified signal name, for
- * instance:
+ * If a name argument starts with "signal::", "signal-after::",
+ * "signal-swapped::" or "signal-swapped-after::" the two following arguments
+ * are used as callback function and data for a signal handler installed on
+ * the #ClutterAnimation object for the specified signal name, for instance:
  *
  * |[
  *
@@ -1849,19 +2064,34 @@ clutter_actor_animate_with_timeline (ClutterActor    *actor,
  *     clutter_actor_hide (actor);
  *   }
  *
- *   clutter_actor_animate (actor, CLUTTER_EASE_IN, 100,
+ *   clutter_actor_animate (actor, CLUTTER_EASE_IN_CUBIC, 100,
  *                          "opacity", 0,
  *                          "signal::completed", on_animation_completed, actor,
  *                          NULL);
  * ]|
  *
+ * or, to automatically destroy an actor at the end of the animation:
+ *
+ * |[
+ *   clutter_actor_animate (actor, CLUTTER_EASE_IN_CUBIC, 100,
+ *                          "opacity", 0,
+ *                          "signal-swapped-after::completed",
+ *                            clutter_actor_destroy,
+ *                            actor,
+ *                          NULL);
+ * ]|
+ *
  * The "signal::" modifier is the equivalent of using g_signal_connect();
  * the "signal-after::" modifier is the equivalent of using
- * g_signal_connect_after(); the "signal-swapped::" modifier is the equivalent
- * of using g_signal_connect_swapped(). The clutter_actor_animate() function
- * will not keep track of multiple connections to the same signal, so it is
- * your responsability to avoid them when calling clutter_actor_animate()
- * multiple times on the same actor.
+ * g_signal_connect_after() or g_signal_connect_data() with the
+ * %G_CONNECT_AFTER; the "signal-swapped::" modifier is the equivalent
+ * of using g_signal_connect_swapped() or g_signal_connect_data() with the
+ * %G_CONNECT_SWAPPED flah; finally, the "signal-swapped-after::" modifier
+ * is the equivalent of using g_signal_connect_data() with both the
+ * %G_CONNECT_AFTER and %G_CONNECT_SWAPPED flags. The clutter_actor_animate()
+ * function will not keep track of multiple connections to the same signal,
+ * so it is your responsability to avoid them when calling
+ * clutter_actor_animate() multiple times on the same actor.
  *
  * Calling this function on an actor that is already being animated
  * will cause the current animation to change with the new final values,
@@ -1869,13 +2099,13 @@ clutter_actor_animate_with_timeline (ClutterActor    *actor,
  *
  * |[
  *   clutter_actor_animate (actor, CLUTTER_LINEAR, 250,
- *                          "width", 100,
- *                          "height", 100,
+ *                          "width", 100.0,
+ *                          "height", 100.0,
  *                          NULL);
  *   clutter_actor_animate (actor, CLUTTER_EASE_IN_CUBIC, 500,
- *                          "x", 100,
- *                          "y", 100,
- *                          "width", 200,
+ *                          "x", 100.0,
+ *                          "y", 100.0,
+ *                          "width", 200.0,
  *                          NULL);
  * ]|
  *
@@ -1883,10 +2113,10 @@ clutter_actor_animate_with_timeline (ClutterActor    *actor,
  *
  * |[
  *   clutter_actor_animate (actor, CLUTTER_EASE_IN_CUBIC, 500,
- *                          "x", 100,
- *                          "y", 100,
- *                          "width", 200,
- *                          "height", 100,
+ *                          "x", 100.0,
+ *                          "y", 100.0,
+ *                          "width", 200.0,
+ *                          "height", 100.0,
  *                          NULL);
  * ]|
  *
@@ -1907,15 +2137,15 @@ clutter_actor_animate_with_timeline (ClutterActor    *actor,
  *                           ClutterActor     *actor)
  *   {
  *     clutter_actor_animate (actor, CLUTTER_EASE_OUT_CUBIC, 250,
- *                            "x", 500,
- *                            "y", 500,
+ *                            "x", 500.0,
+ *                            "y", 500.0,
  *                            NULL);
  *   }
  *
  *     ...
  *     animation = clutter_actor_animate (actor, CLUTTER_EASE_IN_CUBIC, 250,
- *                                        "x", 100,
- *                                        "y", 100,
+ *                                        "x", 100.0,
+ *                                        "y", 100.0,
  *                                        NULL);
  *     g_signal_connect (animation, "completed",
  *                       G_CALLBACK (on_animation_completed),
