@@ -1,7 +1,8 @@
 /* Clutter.
  * An OpenGL based 'interactive canvas' library.
- * Authored By Matthew Allum  <mallum@openedhand.com>
- * Copyright (C) 2006-2007 OpenedHand
+ *
+ * Copyright (C) 2006, 2007, 2008  OpenedHand Ltd
+ * Copyright (C) 2009, 2010  Intel Corp.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,9 +15,13 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ *
+ *
+ * Authored by:
+ *      Matthew Allum <mallum@openedhand.com>
+ *      Emmanuele Bassi <ebassi@linux.intel.com>
  */
 
 #ifdef HAVE_CONFIG_H
@@ -183,7 +188,6 @@ _clutter_backend_x11_events_init (ClutterBackend *backend)
   g_source_add_poll (source, &event_source->event_poll_fd);
   g_source_set_can_recurse (source, TRUE);
   g_source_attach (source, NULL);
-
 }
 
 void
@@ -263,9 +267,10 @@ set_user_time (ClutterBackendX11 *backend_x11,
     }
 }
 
-#if 0 /* See XInput keyboard comment below HAVE_XINPUT */
+#ifdef HAVE_XINPUT
 static void
-convert_xdevicekey_to_xkey (XDeviceKeyEvent *xkev, XEvent *xevent)
+convert_xdevicekey_to_xkey (XDeviceKeyEvent *xkev,
+                            XEvent          *xevent)
 {
   xevent->xany.type = xevent->xkey.type = xkev->type;
   xevent->xkey.serial = xkev->serial;
@@ -410,27 +415,28 @@ event_translate (ClutterBackend *backend,
                  ClutterEvent   *event,
                  XEvent         *xevent)
 {
-  ClutterBackendX11  *backend_x11;
-  ClutterStageX11    *stage_x11;
-  ClutterStage       *stage;
+  ClutterBackendX11 *backend_x11;
+  ClutterStageX11 *stage_x11;
+  ClutterStage *stage;
   ClutterStageWindow *impl;
-  gboolean            res, not_yet_handled = FALSE;
-  Window              xwindow, stage_xwindow;
+  ClutterDeviceManager *manager;
+  gboolean res, not_yet_handled = FALSE;
+  Window xwindow, stage_xwindow;
+  ClutterInputDevice *device;
 
-  backend_x11    = CLUTTER_BACKEND_X11 (backend);
+  backend_x11 = CLUTTER_BACKEND_X11 (backend);
 
   xwindow = xevent->xany.window;
 
   if (backend_x11->event_filters)
     {
-      GSList                *node;
-      ClutterX11EventFilter *filter;
+      GSList *node;
 
       node = backend_x11->event_filters;
 
       while (node)
         {
-          filter = node->data;
+          ClutterX11EventFilter *filter = node->data;
 
           switch (filter->func (xevent, event, filter->data))
             {
@@ -448,19 +454,17 @@ event_translate (ClutterBackend *backend,
         }
     }
 
-  /*
-   * Do further processing only on events for the stage window
-   * (the x11 filters might be getting events for other windows, so do not
-   * mess them about.
+  /* Do further processing only on events for the stage window (the x11
+   * filters might be getting events for other windows, so do not mess
+   * them about.
    */
   stage = clutter_x11_get_stage_from_window (xwindow);
-
   if (stage == NULL)
-      return FALSE;
+    return FALSE;
 
-  impl           = _clutter_stage_get_window (stage);
-  stage_x11      = CLUTTER_STAGE_X11 (impl);
-  stage_xwindow  = xwindow; /* clutter_x11_get_stage_window (stage); */
+  impl = _clutter_stage_get_window (stage);
+  stage_x11 = CLUTTER_STAGE_X11 (impl);
+  stage_xwindow = xwindow; /* clutter_x11_get_stage_window (stage); */
 
   event->any.stage = stage;
 
@@ -468,6 +472,8 @@ event_translate (ClutterBackend *backend,
 
   update_last_event_time (backend_x11, xevent);
 
+  manager = clutter_device_manager_get_default ();
+
   switch (xevent->type)
     {
     case ConfigureNotify:
@@ -479,16 +485,43 @@ event_translate (ClutterBackend *backend,
                         xevent->xconfigure.width,
                         xevent->xconfigure.height);
 
-          stage_x11->xwin_width = xevent->xconfigure.width;
-          stage_x11->xwin_height = xevent->xconfigure.height;
+          /* Queue a relayout - we want glViewport to be called
+           * with the correct values, and this is done in ClutterStage
+           * via _cogl_onscreen_clutter_backend_set_size ().
+           *
+           * We queue a relayout, because if this ConfigureNotify is
+           * in response to a size we set in the application, the
+           * set_size above is essentially a null-op.
+           *
+           * Make sure we do this only when the size has changed,
+           * otherwise we end up relayouting on window moves.
+           */
+          if ((stage_x11->state & CLUTTER_STAGE_STATE_FULLSCREEN) ||
+              (stage_x11->xwin_width != xevent->xconfigure.width) ||
+              (stage_x11->xwin_height != xevent->xconfigure.height))
+          clutter_actor_queue_relayout (CLUTTER_ACTOR (stage));
+
+          /* If we're fullscreened, we want these variables to
+           * represent the size of the window before it was set
+           * to fullscreen.
+           */
+          if (!(stage_x11->state & CLUTTER_STAGE_STATE_FULLSCREEN))
+            {
+              stage_x11->xwin_width = xevent->xconfigure.width;
+              stage_x11->xwin_height = xevent->xconfigure.height;
+            }
+
+          clutter_actor_set_size (CLUTTER_ACTOR (stage),
+                                  xevent->xconfigure.width,
+                                  xevent->xconfigure.height);
+
+          CLUTTER_UNSET_PRIVATE_FLAGS (stage_x11->wrapper,
+                                       CLUTTER_STAGE_IN_RESIZE);
 
           /* the resize process is complete, so we can ask the stage
            * to set up the GL viewport with the new size
            */
-          CLUTTER_SET_PRIVATE_FLAGS (CLUTTER_ACTOR (stage_x11->wrapper),
-                                     CLUTTER_ACTOR_SYNC_MATRICES);
-
-          clutter_actor_queue_relayout (CLUTTER_ACTOR (stage_x11->wrapper));
+          clutter_stage_ensure_viewport (stage);
         }
       res = FALSE;
       break;
@@ -535,6 +568,8 @@ event_translate (ClutterBackend *backend,
                   else
                     stage_x11->state &= ~CLUTTER_STAGE_STATE_FULLSCREEN;
 
+                  stage_x11->fullscreening = fullscreen_set;
+
                   event->type = CLUTTER_STAGE_STATE;
                   event->stage_state.changed_mask =
                     CLUTTER_STAGE_STATE_FULLSCREEN;
@@ -620,13 +655,47 @@ event_translate (ClutterBackend *backend,
 
     case KeyPress:
       event->key.type = event->type = CLUTTER_KEY_PRESS;
+      event->key.device =
+        clutter_device_manager_get_core_device (manager,
+                                                CLUTTER_POINTER_DEVICE);
+
       translate_key_event (backend, event, xevent);
 
       set_user_time (backend_x11, &xwindow, xevent->xkey.time);
       break;
               
     case KeyRelease:
+      /* old-style X11 terminals require that even modern X11 send
+       * KeyPress/KeyRelease pairs when auto-repeating. for this
+       * reason modern(-ish) API like XKB has a way to detect
+       * auto-repeat and do a single KeyRelease at the end of a
+       * KeyPress sequence.
+       *
+       * this check emulates XKB's detectable auto-repeat; we peek
+       * the next event and check if it's a KeyPress for the same key
+       * and timestamp - and then ignore it if it matches the
+       * KeyRelease
+       */
+      if (XPending (xevent->xkey.display))
+        {
+          XEvent next_event;
+
+          XPeekEvent (xevent->xkey.display, &next_event);
+
+          if (next_event.type == KeyPress &&
+              next_event.xkey.keycode == xevent->xkey.keycode &&
+              next_event.xkey.time == xevent->xkey.time)
+            {
+              res = FALSE;
+              break;
+            }
+        }
+
       event->key.type = event->type = CLUTTER_KEY_RELEASE;
+      event->key.device =
+        clutter_device_manager_get_core_device (manager,
+                                                CLUTTER_KEYBOARD_DEVICE);
+
       translate_key_event (backend, event, xevent);
       break;
 
@@ -639,239 +708,288 @@ event_translate (ClutterBackend *backend,
   /* Input device event handling.. */
   if (not_yet_handled)
     {
-      if (!clutter_x11_has_xinput ())
+      device = clutter_device_manager_get_core_device (manager,
+                                                       CLUTTER_POINTER_DEVICE);
+
+      /* Regular X event */
+      switch (xevent->type)
         {
-          /* Regular X event */
-          switch (xevent->type)
+        case ButtonPress:
+          switch (xevent->xbutton.button)
             {
-			/* KeyPress / KeyRelease should reside here if XInput
-             * worked properly
-             */
-            case ButtonPress:
-              switch (xevent->xbutton.button)
-                {
-                case 4: /* up */
-                case 5: /* down */
-                case 6: /* left */
-                case 7: /* right */
-                  event->scroll.type = event->type = CLUTTER_SCROLL;
-                  
-                  if (xevent->xbutton.button == 4)
-                    event->scroll.direction = CLUTTER_SCROLL_UP;
-                  else if (xevent->xbutton.button == 5)
-                    event->scroll.direction = CLUTTER_SCROLL_DOWN;
-                  else if (xevent->xbutton.button == 6)
-                    event->scroll.direction = CLUTTER_SCROLL_LEFT;
-                  else
-                    event->scroll.direction = CLUTTER_SCROLL_RIGHT;
-                  
-                  event->scroll.time = xevent->xbutton.time;
-                  event->scroll.x = xevent->xbutton.x;
-                  event->scroll.y = xevent->xbutton.y;
-                  event->scroll.modifier_state = xevent->xbutton.state;
-                  
-                  break;
-                default:
-                  event->button.type = event->type = CLUTTER_BUTTON_PRESS;
-                  event->button.time = xevent->xbutton.time;
-                  event->button.x = xevent->xbutton.x;
-                  event->button.y = xevent->xbutton.y;
-                  event->button.modifier_state = xevent->xbutton.state;
-                  event->button.button = xevent->xbutton.button;
-                  
-                  break;
-                }
-              
-              set_user_time (backend_x11, &xwindow, event->button.time);
+            case 4: /* up */
+            case 5: /* down */
+            case 6: /* left */
+            case 7: /* right */
+              event->scroll.type = event->type = CLUTTER_SCROLL;
+
+              if (xevent->xbutton.button == 4)
+                event->scroll.direction = CLUTTER_SCROLL_UP;
+              else if (xevent->xbutton.button == 5)
+                event->scroll.direction = CLUTTER_SCROLL_DOWN;
+              else if (xevent->xbutton.button == 6)
+                event->scroll.direction = CLUTTER_SCROLL_LEFT;
+              else
+                event->scroll.direction = CLUTTER_SCROLL_RIGHT;
+
+              event->scroll.time = xevent->xbutton.time;
+              event->scroll.x = xevent->xbutton.x;
+              event->scroll.y = xevent->xbutton.y;
+              event->scroll.modifier_state = xevent->xbutton.state;
+              event->scroll.device = device;
               break;
-              
-            case ButtonRelease:
-              /* scroll events don't have a corresponding release */
-              if (xevent->xbutton.button == 4 ||
-                  xevent->xbutton.button == 5 ||
-                  xevent->xbutton.button == 6 ||
-                  xevent->xbutton.button == 7)
-                {
-                  res = FALSE;
-                  break;
-                }
-              
-              event->button.type = event->type = CLUTTER_BUTTON_RELEASE;
+
+            default:
+              event->button.type = event->type = CLUTTER_BUTTON_PRESS;
               event->button.time = xevent->xbutton.time;
               event->button.x = xevent->xbutton.x;
               event->button.y = xevent->xbutton.y;
               event->button.modifier_state = xevent->xbutton.state;
               event->button.button = xevent->xbutton.button;
+              event->button.device = device;
               break;
-              
-            case MotionNotify:
-              event->motion.type = event->type = CLUTTER_MOTION;
-              event->motion.time = xevent->xmotion.time;
-              event->motion.x = xevent->xmotion.x;
-              event->motion.y = xevent->xmotion.y;
-              event->motion.modifier_state = xevent->xmotion.state;
-              break;
+            }
 
-            case EnterNotify:
-              /* Convert enter notifies to motion events because X
-                 doesn't emit the corresponding motion notify */
-              event->motion.type = event->type = CLUTTER_MOTION;
-              event->motion.time = xevent->xcrossing.time;
-              event->motion.x = xevent->xcrossing.x;
-              event->motion.y = xevent->xcrossing.y;
-              event->motion.modifier_state = xevent->xcrossing.state;
-              break;
+          set_user_time (backend_x11, &xwindow, event->button.time);
 
-            case LeaveNotify:
-              event->crossing.type = event->type = CLUTTER_LEAVE;
-              event->crossing.time = xevent->xcrossing.time;
-              event->crossing.x = xevent->xcrossing.x;
-              event->crossing.y = xevent->xcrossing.y;
+          res = TRUE;
+          break;
+
+        case ButtonRelease:
+          /* scroll events don't have a corresponding release */
+          if (xevent->xbutton.button == 4 ||
+              xevent->xbutton.button == 5 ||
+              xevent->xbutton.button == 6 ||
+              xevent->xbutton.button == 7)
+            {
+              res = FALSE;
+              goto out;
+            }
+
+          event->button.type = event->type = CLUTTER_BUTTON_RELEASE;
+          event->button.time = xevent->xbutton.time;
+          event->button.x = xevent->xbutton.x;
+          event->button.y = xevent->xbutton.y;
+          event->button.modifier_state = xevent->xbutton.state;
+          event->button.button = xevent->xbutton.button;
+          event->button.device = device;
+
+          res = TRUE;
+          break;
+
+        case MotionNotify:
+          event->motion.type = event->type = CLUTTER_MOTION;
+          event->motion.time = xevent->xmotion.time;
+          event->motion.x = xevent->xmotion.x;
+          event->motion.y = xevent->xmotion.y;
+          event->motion.modifier_state = xevent->xmotion.state;
+          event->motion.device = device;
+
+          res = TRUE;
+          break;
+
+        case EnterNotify:
+          /* we know that we are entering the stage here */
+          _clutter_input_device_set_stage (device, stage);
+          CLUTTER_NOTE (EVENT, "Entering the stage");
+
+          /* Convert enter notifies to motion events because X
+             doesn't emit the corresponding motion notify */
+          event->motion.type = event->type = CLUTTER_MOTION;
+          event->motion.time = xevent->xcrossing.time;
+          event->motion.x = xevent->xcrossing.x;
+          event->motion.y = xevent->xcrossing.y;
+          event->motion.modifier_state = xevent->xcrossing.state;
+          event->motion.source = CLUTTER_ACTOR (stage);
+          event->motion.device = device;
+
+          res = TRUE;
+          break;
+
+        case LeaveNotify:
+          if (device->stage == NULL)
+            {
+              CLUTTER_NOTE (EVENT,
+                            "Discarding LeaveNotify for ButtonRelease "
+                            "event off-stage");
+              res = FALSE;
+              goto out;
+            }
+
+          /* we know that we are leaving the stage here */
+          _clutter_input_device_set_stage (device, NULL);
+          CLUTTER_NOTE (EVENT, "Leaving the stage (time:%u)",
+                        event->crossing.time);
+
+          event->crossing.type = event->type = CLUTTER_LEAVE;
+          event->crossing.time = xevent->xcrossing.time;
+          event->crossing.x = xevent->xcrossing.x;
+          event->crossing.y = xevent->xcrossing.y;
+          event->crossing.source = CLUTTER_ACTOR (stage);
+          event->crossing.device = device;
+          res = TRUE;
+          break;
+
+        default:
+          res = FALSE;
+          break;
+        }
+    }
+
+    /* XInput fun...*/
+  if (!res && clutter_x11_has_xinput ())
+    {
+#ifdef HAVE_XINPUT
+      int *ev_types = backend_x11->event_types;
+      int button_press, button_release;
+      int key_press, key_release;
+      int motion_notify;
+
+      button_press   = ev_types[CLUTTER_X11_XINPUT_BUTTON_PRESS_EVENT];
+      button_release = ev_types[CLUTTER_X11_XINPUT_BUTTON_RELEASE_EVENT];
+      motion_notify  = ev_types[CLUTTER_X11_XINPUT_MOTION_NOTIFY_EVENT];
+      key_press      = ev_types[CLUTTER_X11_XINPUT_KEY_PRESS_EVENT];
+      key_release    = ev_types[CLUTTER_X11_XINPUT_KEY_RELEASE_EVENT];
+
+      CLUTTER_NOTE (EVENT, "XInput event type: %d", xevent->type);
+
+      if (xevent->type == button_press)
+        {
+          XDeviceButtonEvent *xbev = (XDeviceButtonEvent *) xevent;
+
+          device = _clutter_x11_get_device_for_xid (xbev->deviceid);
+          _clutter_input_device_set_stage (device, stage);
+
+          CLUTTER_NOTE (EVENT,
+                        "XI ButtonPress for %li ('%s') at %d, %d",
+                        xbev->deviceid,
+                        device->device_name,
+                        xbev->x,
+                        xbev->y);
+
+          switch (xbev->button)
+            {
+            case 4:
+            case 5:
+            case 6:
+            case 7:
+              event->scroll.type = event->type = CLUTTER_SCROLL;
+
+              if (xbev->button == 4)
+                event->scroll.direction = CLUTTER_SCROLL_UP;
+              else if (xbev->button == 5)
+                event->scroll.direction = CLUTTER_SCROLL_DOWN;
+              else if (xbev->button == 6)
+                event->scroll.direction = CLUTTER_SCROLL_LEFT;
+              else
+                event->scroll.direction = CLUTTER_SCROLL_RIGHT;
+
+              event->scroll.time = xbev->time;
+              event->scroll.x = xbev->x;
+              event->scroll.y = xbev->y;
+              event->scroll.modifier_state = xbev->state;
+              event->scroll.device = device;
               break;
 
             default:
-              /* ignore every other event */
-              res = FALSE;
-              break;
-            }
-        }
-      else
-        {  /* XInput fun.. Needs clean up. */
-#ifdef HAVE_XINPUT
-          int *ev_types = backend_x11->event_types;
-
-          CLUTTER_NOTE (EVENT, "XInput event type: %d", xevent->type);
-
-          if (xevent->type == ev_types [CLUTTER_X11_XINPUT_BUTTON_PRESS_EVENT])
-            {
-              XDeviceButtonEvent *xbev = (XDeviceButtonEvent *) xevent;
-
-              CLUTTER_NOTE (EVENT, "XINPUT Button press event for %li at %d, %d",
-                            xbev->deviceid,
-                            xbev->x,
-                            xbev->y);
-
-              switch (xbev->button)
-                {
-                case 4:
-                case 5:
-                case 6:
-                case 7:
-                  event->scroll.type = event->type = CLUTTER_SCROLL;
-                  
-                  if (xbev->button == 4)
-                    event->scroll.direction = CLUTTER_SCROLL_UP;
-                  else if (xbev->button == 5)
-                    event->scroll.direction = CLUTTER_SCROLL_DOWN;
-                  else if (xbev->button == 6)
-                    event->scroll.direction = CLUTTER_SCROLL_LEFT;
-                  else
-                    event->scroll.direction = CLUTTER_SCROLL_RIGHT;
-                  
-                  event->scroll.time = xbev->time;
-                  event->scroll.x = xbev->x;
-                  event->scroll.y = xbev->y;
-                  event->scroll.modifier_state = xbev->state;
-                  event->scroll.device = _clutter_x11_get_device_for_xid (xbev->deviceid);
-                  break;
-
-                default:
-                  event->button.type = event->type = CLUTTER_BUTTON_PRESS;
-                  event->button.time = xbev->time;
-                  event->button.x = xbev->x;
-                  event->button.y = xbev->y;
-                  event->button.modifier_state = xbev->state;
-                  event->button.button = xbev->button;
-                  event->button.device = _clutter_x11_get_device_for_xid (xbev->deviceid);
-                  break;
-                }
-
-              set_user_time (backend_x11, &xwindow, xbev->time);
-            } 
-          else if (xevent->type 
-                        == ev_types[CLUTTER_X11_XINPUT_BUTTON_RELEASE_EVENT])
-            {
-              XDeviceButtonEvent *xbev = (XDeviceButtonEvent *)xevent;
-
-              CLUTTER_NOTE (EVENT, "XINPUT Button release event for %li at %d, %d",
-                            xbev->deviceid,
-                            xbev->x,
-                            xbev->y);
-
-              /* scroll events don't have a corresponding release */
-              if (xbev->button == 4 ||
-                  xbev->button == 5 ||
-                  xbev->button == 6 ||
-                  xbev->button == 7)
-                {
-                  return FALSE;
-                }
-
-              event->button.type = event->type = CLUTTER_BUTTON_RELEASE;
+              event->button.type = event->type = CLUTTER_BUTTON_PRESS;
               event->button.time = xbev->time;
               event->button.x = xbev->x;
               event->button.y = xbev->y;
               event->button.modifier_state = xbev->state;
               event->button.button = xbev->button;
-              event->button.device = _clutter_x11_get_device_for_xid (xbev->deviceid);
-            } 
-          else if (xevent->type 
-                       == ev_types [CLUTTER_X11_XINPUT_MOTION_NOTIFY_EVENT]) 
-            {
-              XDeviceMotionEvent *xmev = (XDeviceMotionEvent *)xevent;
-
-              CLUTTER_NOTE(EVENT, "XINPUT Motion event for %li at %d, %d",
-                           xmev->deviceid,
-                           xmev->x,
-                           xmev->y);
-
-              event->motion.type = event->type = CLUTTER_MOTION;
-              event->motion.time = xmev->time;
-              event->motion.x = xmev->x;
-              event->motion.y = xmev->y;
-              event->motion.modifier_state = xmev->state;
-              event->motion.device = _clutter_x11_get_device_for_xid (xmev->deviceid);
-            } 
-#if 0
-        /* the Xinput handling of key presses/releases disabled for now since
-         * it makes keyrepeat, and key presses and releases outside the window
-         * not generate events even when the window has focus
-         */
-
-          else if (xevent->type 
-                        == ev_types [CLUTTER_X11_XINPUT_KEY_PRESS_EVENT]) 
-            {
-              XEvent xevent_converted;
-              XDeviceKeyEvent *xkev = (XDeviceKeyEvent *)xevent;
-              
-              convert_xdevicekey_to_xkey (xkev, &xevent_converted);
-
-              event->key.type = event->type = CLUTTER_KEY_PRESS;
-              translate_key_event (backend, event, &xevent_converted);
-
-              set_user_time (backend_x11, &xwindow, xkev->time);
-            } 
-          else if (xevent->type 
-                   == ev_types [CLUTTER_X11_XINPUT_KEY_RELEASE_EVENT]) 
-            {
-              XEvent xevent_converted;
-              XDeviceKeyEvent *xkev = (XDeviceKeyEvent *)xevent;
-              
-              convert_xdevicekey_to_xkey (xkev, &xevent_converted);
-
-              event->key.type = event->type = CLUTTER_KEY_RELEASE;
-              translate_key_event (backend, event, &xevent_converted);
+              event->button.device = device;
+              break;
             }
-#endif
-          else 
-#endif /* HAVE_XINPUT */
+
+          set_user_time (backend_x11, &xwindow, xbev->time);
+
+          res = TRUE;
+        }
+      else if (xevent->type == button_release)
+        {
+          XDeviceButtonEvent *xbev = (XDeviceButtonEvent *)xevent;
+
+          device = _clutter_x11_get_device_for_xid (xbev->deviceid);
+          _clutter_input_device_set_stage (device, stage);
+
+          CLUTTER_NOTE (EVENT, "XI ButtonRelease for %li ('%s') at %d, %d",
+                        xbev->deviceid,
+                        device->device_name,
+                        xbev->x,
+                        xbev->y);
+
+          /* scroll events don't have a corresponding release */
+          if (xbev->button == 4 ||
+              xbev->button == 5 ||
+              xbev->button == 6 ||
+              xbev->button == 7)
             {
-              CLUTTER_NOTE (EVENT, "Uknown Event");
               res = FALSE;
+              goto out;
             }
+
+          event->button.type = event->type = CLUTTER_BUTTON_RELEASE;
+          event->button.time = xbev->time;
+          event->button.x = xbev->x;
+          event->button.y = xbev->y;
+          event->button.modifier_state = xbev->state;
+          event->button.button = xbev->button;
+          event->button.device = device;
+
+          res = TRUE;
+        }
+      else if (xevent->type == motion_notify)
+        {
+          XDeviceMotionEvent *xmev = (XDeviceMotionEvent *)xevent;
+
+          device = _clutter_x11_get_device_for_xid (xmev->deviceid);
+          _clutter_input_device_set_stage (device, stage);
+
+          CLUTTER_NOTE (EVENT, "XI Motion for %li ('%s') at %d, %d",
+                        xmev->deviceid,
+                        device->device_name,
+                        xmev->x,
+                        xmev->y);
+
+          event->motion.type = event->type = CLUTTER_MOTION;
+          event->motion.time = xmev->time;
+          event->motion.x = xmev->x;
+          event->motion.y = xmev->y;
+          event->motion.modifier_state = xmev->state;
+          event->motion.device = device;
+
+          res = TRUE;
+        }
+      else if (xevent->type == key_press || xevent->type == key_release)
+        {
+          /* the XInput 1.x handling of key presses/releases is broken:
+           * it makes key repeat, key presses and releases outside the
+           * window not generate events even when the window has focus
+           */
+          XDeviceKeyEvent *xkev = (XDeviceKeyEvent *) xevent;
+          XEvent xevent_converted;
+
+          convert_xdevicekey_to_xkey (xkev, &xevent_converted);
+
+          event->key.type = event->type = (xevent->type == key_press)
+                                          ? CLUTTER_KEY_PRESS
+                                          : CLUTTER_KEY_RELEASE;
+
+          translate_key_event (backend, event, &xevent_converted);
+
+          if (xevent->type == key_press)
+            set_user_time (backend_x11, &xwindow, xkev->time);
+        }
+      else
+#endif /* HAVE_XINPUT */
+        {
+          CLUTTER_NOTE (EVENT, "Uknown Event");
+          res = FALSE;
         }
     }
 
+out:
   return res;
 }
 
@@ -879,6 +997,8 @@ static void
 events_queue (ClutterBackend *backend)
 {
   ClutterBackendX11 *backend_x11 = CLUTTER_BACKEND_X11 (backend);
+  ClutterBackendX11Class *backend_x11_class =
+    CLUTTER_BACKEND_X11_GET_CLASS (backend_x11);
   ClutterEvent      *event;
   Display           *xdisplay = backend_x11->xdpy;
   XEvent             xevent;
@@ -890,6 +1010,9 @@ events_queue (ClutterBackend *backend)
     {
       XNextEvent (xdisplay, &xevent);
 
+      if (backend_x11_class->handle_event (backend_x11, &xevent))
+        continue;
+
       event = clutter_event_new (CLUTTER_NOTHING);
 
       if (event_translate (backend, event, &xevent))
