@@ -4,9 +4,14 @@
 
 #include <clutter/clutter.h>
 
+#ifndef _MSC_VER
 #include <unistd.h> 		/* for sleep(), used for screenshots */
+#endif
 #include <stdlib.h>
-#include "math.h"
+#ifdef _MSC_VER
+#define _USE_MATH_DEFINES
+#endif
+#include <math.h>
 
 #define PETAL_MIN 20
 #define PETAL_VAR 40
@@ -132,20 +137,24 @@ make_flower_actor (void)
   return ctex;
 }
 
-gboolean
-tick (gpointer data)
+static void
+tick (ClutterTimeline *timeline,
+      gint             msecs,
+      gpointer         data)
 {
   Flower **flowers = (Flower**)data;
   gint i = 0;
 
-  for (i=0; i< N_FLOWERS; i++)
+  for (i = 0; i < N_FLOWERS; i++)
     {
+      ClutterActor *stage;
+
       flowers[i]->y   += flowers[i]->v;
       flowers[i]->rot += flowers[i]->rv;
 
-      if (flowers[i]->y > (gint)clutter_actor_get_height
-	                        (clutter_stage_get_default ()))
-	    flowers[i]->y = -clutter_actor_get_height (flowers[i]->ctex);
+      stage = clutter_stage_get_default ();
+      if (flowers[i]->y > (gint) clutter_actor_get_height (stage))
+        flowers[i]->y = -clutter_actor_get_height (flowers[i]->ctex);
 
       clutter_actor_set_position (flowers[i]->ctex,
 				  flowers[i]->x, flowers[i]->y);
@@ -157,19 +166,16 @@ tick (gpointer data)
                                   clutter_actor_get_height (flowers[i]->ctex)/2,
                                   0);
     }
-
-  return TRUE;
 }
 
-void foo(void) { g_usleep(10000000); }
-
 int
 test_clutter_cairo_flowers_main (int argc, char **argv)
 {
-  int           i;
-  ClutterActor *stage;
-  ClutterColor  stage_color = { 0x0, 0x0, 0x0, 0xff };
-  Flower       *flowers[N_FLOWERS];
+  int              i;
+  ClutterActor    *stage;
+  ClutterColor     stage_color = { 0x0, 0x0, 0x0, 0xff };
+  ClutterTimeline *timeline;
+  Flower          *flowers[N_FLOWERS];
 
   srand (time (NULL));
 
@@ -196,10 +202,17 @@ test_clutter_cairo_flowers_main (int argc, char **argv)
 				  flowers[i]->x, flowers[i]->y);
     }
 
-  g_timeout_add (50, tick, flowers);
+  /* Create a timeline to manage animation */
+  timeline = clutter_timeline_new (6000);
+  clutter_timeline_set_loop (timeline, TRUE);
+
+  /* fire a callback for frame change */
+  g_signal_connect (timeline, "new-frame", G_CALLBACK (tick), flowers);
 
   clutter_actor_show (stage);
 
+  clutter_timeline_start (timeline);
+
   g_signal_connect (stage, "key-press-event",
 		    G_CALLBACK (clutter_main_quit),
 		    NULL);
